@@ -20,10 +20,10 @@ class TimestreamLoader(TimestreamReader):
         table_name: str = None,
         session: boto3.Session = None,
         current_time_ms: int = None,
-        logger: logging.Logger = None
+        logger: logging.Logger = None,
     ):
-        assert database_name, 'Must specify a database_name'
-        assert table_name, 'Must specify table_name'
+        assert database_name, "Must specify a database_name"
+        assert table_name, "Must specify table_name"
         super().__init__(session, current_time_ms, logger)
         self.__database_name = database_name
         self.__table_name = table_name
@@ -36,28 +36,20 @@ class TimestreamLoader(TimestreamReader):
     def table_name(self) -> str:
         return self.__table_name
 
-    def does_database_exist(
-        self,
-        database_name: str = None,
-        raise_err: bool = False
-    ) -> bool:
-        '''Overridden to maintain backwards compatibility'''
+    def does_database_exist(self, database_name: str = None, raise_err: bool = False) -> bool:
+        """Overridden to maintain backwards compatibility"""
         return super().does_database_exist(
-            database_name=database_name or self.__database_name,
-            raise_err=raise_err
+            database_name=database_name or self.__database_name, raise_err=raise_err
         )
 
     def does_table_exist(
-        self,
-        database_name: str = None,
-        table_name: str = None,
-        raise_err: bool = False
+        self, database_name: str = None, table_name: str = None, raise_err: bool = False
     ) -> bool:
-        '''Overridden to maintain backwards compatibility'''
+        """Overridden to maintain backwards compatibility"""
         return super().does_table_exist(
             database_name=database_name or self.__database_name,
             table_name=table_name or self.__table_name,
-            raise_err=raise_err
+            raise_err=raise_err,
         )
 
     def load(
@@ -71,12 +63,12 @@ class TimestreamLoader(TimestreamReader):
         ignore_parse_error: bool = True,
         max_batch_size: int = DEFAULT_MAX_BATCH_SIZE,
         min_common_dimensions: int = DEFAULT_MIN_COMMON_DIMENSIONS,
-        is_multi: bool = True
+        is_multi: bool = True,
     ) -> dict:
-        '''
+        """
         Performs a Timestream load. Validates the input and parses the data into Timestream records.
         Returns the result of the load after writing to Timestream.
-        '''
+        """
         return self.__load(
             data,
             capture_time,
@@ -88,7 +80,7 @@ class TimestreamLoader(TimestreamReader):
             False,
             max_batch_size,
             min_common_dimensions,
-            is_multi
+            is_multi,
         ).to_dict()
 
     def load_preview(
@@ -102,13 +94,13 @@ class TimestreamLoader(TimestreamReader):
         ignore_parse_error: bool = True,
         max_batch_size: int = DEFAULT_MAX_BATCH_SIZE,
         min_common_dimensions: int = DEFAULT_MIN_COMMON_DIMENSIONS,
-        is_multi: bool = True
+        is_multi: bool = True,
     ) -> dict:
-        '''
+        """
         Performs a dry run of a Timestream load, including validating the input values and parsing data
         into Timestream records.
         Returns the result of the load before writing to Timestream.
-        '''
+        """
         return self.__load(
             data,
             capture_time,
@@ -120,7 +112,7 @@ class TimestreamLoader(TimestreamReader):
             True,
             max_batch_size,
             min_common_dimensions,
-            is_multi
+            is_multi,
         ).to_dict()
 
     def __load(
@@ -135,9 +127,9 @@ class TimestreamLoader(TimestreamReader):
         preview: bool = False,
         max_batch_size: int = DEFAULT_MAX_BATCH_SIZE,
         min_common_dimensions: int = DEFAULT_MIN_COMMON_DIMENSIONS,
-        is_multi: bool = True
+        is_multi: bool = True,
     ) -> TimestreamLoadResult:
-        self._logger.info(f'Loading data with {len(data)} elements')
+        self._logger.info(f"Loading data with {len(data)} elements")
 
         # Parse incoming data into TimestreamRecords
         result = self.__parse_data(data, upsert, ignore_parse_error, is_multi)
@@ -145,51 +137,42 @@ class TimestreamLoader(TimestreamReader):
             return result
 
         # Batch the accepted records by common dimensions
-        self._logger.info('Starting batching...')
+        self._logger.info("Starting batching...")
         batches = batch_records(
             result.accepted_records,
             max_batch_size=max_batch_size,
             min_common_dimensions=min_common_dimensions,
-            logger=self._logger
+            logger=self._logger,
         )
-        assert isinstance(batches, list) and len(batches) > 0, 'Batches must be a non-empty list'
-        assert all(len(b.records) > 0 for b in batches), "Not all batches have records (shouldn't be possible)"
-        self._logger.info('Batching complete')
+        assert isinstance(batches, list) and len(batches) > 0, "Batches must be a non-empty list"
+        assert all(
+            len(b.records) > 0 for b in batches
+        ), "Not all batches have records (shouldn't be possible)"
+        self._logger.info("Batching complete")
 
         # Write records in batches to Timestream
         result = TimestreamLoadResult()
         num_batches = len(batches)
         for i in range(num_batches):
-            self._logger.info(f'Writing batch {i+1}/{num_batches}')
+            self._logger.info(f"Writing batch {i+1}/{num_batches}")
             b = batches[i]
             dimensions = b.common_dimensions
             if source:
-                dimensions.add_dimension('source', source, overwrite=True)
+                dimensions.add_dimension("source", source, overwrite=True)
             if b.records and upsert:
                 accepted_records, rejected_records = self.__write_upsert_records(
-                    dimensions,
-                    b.records,
-                    source,
-                    database_name,
-                    table_name,
-                    is_multi
+                    dimensions, b.records, source, database_name, table_name, is_multi
                 )
             elif b.records:
                 accepted_records, rejected_records = self.__write_insert_records(
-                    dimensions,
-                    b.records,
-                    source,
-                    capture_time,
-                    database_name,
-                    table_name,
-                    is_multi
+                    dimensions, b.records, source, capture_time, database_name, table_name, is_multi
                 )
 
             if accepted_records:
                 result.add_accepted_records(accepted_records)
             if rejected_records:
                 result.add_rejected_records(rejected_records)
-                result.error = Exception('Some records were rejected by Timestream')
+                result.error = Exception("Some records were rejected by Timestream")
         return result
 
     def __write_insert_records(
@@ -200,40 +183,33 @@ class TimestreamLoader(TimestreamReader):
         capture_time: datetime,
         database_name: str = None,
         table_name: str = None,
-        is_multi: bool = True
+        is_multi: bool = True,
     ) -> tuple:
-        self._logger.debug(f'Writing insert records with {len(records)} records')
-        assert capture_time, 'A capture time needs to be provided'
+        self._logger.debug(f"Writing insert records with {len(records)} records")
+        assert capture_time, "A capture time needs to be provided"
         capture_time_ms = self._dt_to_ms(capture_time)
         accepted_records, rejected_records = records, []
         try:
-            common_att = {
-                'MeasureValueType': 'DOUBLE',
-                'Time': str(capture_time_ms),
-                'Version': 1
-            }
+            common_att = {"MeasureValueType": "DOUBLE", "Time": str(capture_time_ms), "Version": 1}
             if is_multi:
-                common_att['MeasureName'] = source or 'multi-measure'
-                common_att['MeasureValueType'] = 'MULTI'
-                if 'source' in common_dimensions.get_dimension_keys():
+                common_att["MeasureName"] = source or "multi-measure"
+                common_att["MeasureValueType"] = "MULTI"
+                if "source" in common_dimensions.get_dimension_keys():
                     # Remove the source dimension
                     # Source becomes MeasureName as stated above
-                    common_dimensions.remove_dimension('source')
-                common_att['Dimensions'] = common_dimensions.loadable()
+                    common_dimensions.remove_dimension("source")
+                common_att["Dimensions"] = common_dimensions.loadable()
             else:
-                common_att['Dimensions'] = common_dimensions.loadable()
+                common_att["Dimensions"] = common_dimensions.loadable()
 
             _ = self._timestream_write_client.write_records(
                 DatabaseName=database_name or self.__database_name,
                 TableName=table_name or self.__table_name,
                 CommonAttributes=common_att,
-                Records=[
-                    r.loadable(omit_dimensions=common_dimensions)
-                    for r in records
-                ]
+                Records=[r.loadable(omit_dimensions=common_dimensions) for r in records],
             )
         except ClientError as e:
-            self._logger.error(f'Failed to write insert records: {e}')
+            self._logger.error(f"Failed to write insert records: {e}")
             accepted_records, rejected_records = self.__process_client_error(e, records, is_multi)
         return accepted_records, rejected_records
 
@@ -244,76 +220,71 @@ class TimestreamLoader(TimestreamReader):
         source: str,
         database_name: str = None,
         table_name: str = None,
-        is_multi: bool = True
+        is_multi: bool = True,
     ) -> tuple:
-        self._logger.debug(f'Writing upsert records with {len(records)} records')
+        self._logger.debug(f"Writing upsert records with {len(records)} records")
         accepted_records, rejected_records = records, []
         try:
-            common_att = {
-                'MeasureValueType': 'DOUBLE',
-                'Version': self._current_time_ms
-            }
+            common_att = {"MeasureValueType": "DOUBLE", "Version": self._current_time_ms}
             if is_multi:
-                common_att['MeasureName'] = source or 'multi-measure'
-                common_att['MeasureValueType'] = 'MULTI'
-                if 'source' in common_dimensions.get_dimension_keys():
+                common_att["MeasureName"] = source or "multi-measure"
+                common_att["MeasureValueType"] = "MULTI"
+                if "source" in common_dimensions.get_dimension_keys():
                     # Remove the source dimension
                     # Source becomes MeasureName as stated above
-                    common_dimensions.remove_dimension('source')
-                common_att['Dimensions'] = common_dimensions.loadable()
+                    common_dimensions.remove_dimension("source")
+                common_att["Dimensions"] = common_dimensions.loadable()
             else:
-                common_att['Dimensions'] = common_dimensions.loadable()
-                    
+                common_att["Dimensions"] = common_dimensions.loadable()
+
             _ = self._timestream_write_client.write_records(
                 DatabaseName=database_name or self.__database_name,
                 TableName=table_name or self.__table_name,
                 CommonAttributes=common_att,
-                Records=[
-                    r.loadable(omit_dimensions=common_dimensions)
-                    for r in records
-                ]
+                Records=[r.loadable(omit_dimensions=common_dimensions) for r in records],
             )
         except ClientError as e:
-            self._logger.error(f'Failed to write upsert records: {e}')
+            self._logger.error(f"Failed to write upsert records: {e}")
             accepted_records, rejected_records = self.__process_client_error(e, records, is_multi)
         return accepted_records, rejected_records
 
-    def __process_client_error(self, e: ClientError, records: List[dict], is_multi: bool = False) -> tuple:
-        '''
+    def __process_client_error(
+        self, e: ClientError, records: List[dict], is_multi: bool = False
+    ) -> tuple:
+        """
         Processes a boto ClientError from a Timestream write operation. Logs error messages
         for each rejected Timestream record and returns 2 lists:
           1. accepted_records: the records that were successfully written to Timestream
           2. rejected_records: the records that Timestream rejected, and the reasons why
-        '''
+        """
         rejected_records, rejected_indices = [], []
-        for rejection_data in e.response.get('RejectedRecords', []):
+        for rejection_data in e.response.get("RejectedRecords", []):
             if is_multi:
                 rejected_record = RejectedTimestreamRecordMulti(
-                    record=records[rejection_data['RecordIndex']],
-                    error=Exception(rejection_data['Reason'])
+                    record=records[rejection_data["RecordIndex"]],
+                    error=Exception(rejection_data["Reason"]),
                 )
-            
+
             else:
                 rejected_record = RejectedTimestreamRecord(
-                    record=records[rejection_data['RecordIndex']],
-                    error=Exception(rejection_data['Reason'])
+                    record=records[rejection_data["RecordIndex"]],
+                    error=Exception(rejection_data["Reason"]),
                 )
-            self._logger.error(f'Rejected record:\n{rejected_record}')
+            self._logger.error(f"Rejected record:\n{rejected_record}")
             rejected_records.append(rejected_record)
-            rejected_indices.append(rejection_data['RecordIndex'])
+            rejected_indices.append(rejection_data["RecordIndex"])
 
-        accepted_records = [
-            records[i] for i in range(len(records))
-            if i not in rejected_indices
-        ]
+        accepted_records = [records[i] for i in range(len(records)) if i not in rejected_indices]
         return accepted_records, rejected_records
 
-    def __parse_data(self, data: List[dict], upsert: bool, ignore_parse_error: bool, is_multi: bool) -> TimestreamLoadResult:
-        '''
+    def __parse_data(
+        self, data: List[dict], upsert: bool, ignore_parse_error: bool, is_multi: bool
+    ) -> TimestreamLoadResult:
+        """
         Parses the given data and returns a TimestreamLoadResult that contains:
           1. Successfully parsed TimestreamRecords
           2. Invalid items that failed to parse into TimestreamRecords (and the errors for why they failed to parse)
-        '''
+        """
         # TODO: detect duplicates
         result = TimestreamLoadResult()
         if not data:
@@ -324,14 +295,16 @@ class TimestreamLoader(TimestreamReader):
             try:
                 records_from_item = self.__parse_item(item, upsert, ignore_parse_error, is_multi)
             except AssertionError as e:
-                result.error = Exception('Some items could not be parsed into Timestream records')
+                result.error = Exception("Some items could not be parsed into Timestream records")
                 result.add_invalid_item(item, e)
                 continue
             result.add_accepted_records(records_from_item)
         return result
 
-    def __parse_item(self, item: dict, upsert: bool, ignore_parse_error: bool, is_multi: bool) -> List[TimestreamRecord]:
-        '''
+    def __parse_item(
+        self, item: dict, upsert: bool, ignore_parse_error: bool, is_multi: bool
+    ) -> List[TimestreamRecord]:
+        """
         Parses a single dictionary into Timestream records. A single dictionary may break down into
         multiple Timestream records if there are multiple numerical values in the dictionary.
 
@@ -339,7 +312,7 @@ class TimestreamLoader(TimestreamReader):
         If "ignore_parse_error" is true, this will attempt to ignore all values of the item
         that cannot be converted into either a Timestream record or dimension (rather than
         raising an AssertionError).
-        '''
+        """
         if not item:
             return []
         assert type(item) is dict, f'Invalid item with type "{type(item)}", expecting a dictionary'
@@ -351,7 +324,7 @@ class TimestreamLoader(TimestreamReader):
         mstime = None if not mstimes else mstimes[0]
         if upsert:
             assert mstime is not None, f'To perform an upsert, item must have a "time" field'
-            assert len(mstimes) < 2, f'Item has multiple time fields'
+            assert len(mstimes) < 2, f"Item has multiple time fields"
         else:
             assert mstime is None, f'To perform an insert, item cannot specify a "time" field'
 
@@ -360,19 +333,21 @@ class TimestreamLoader(TimestreamReader):
             if v is None or is_time_key(k):
                 continue
             elif is_version_key(k):
-                raise AssertionError('Field "version" is a protected field and cannot exist on an item')
+                raise AssertionError(
+                    'Field "version" is a protected field and cannot exist on an item'
+                )
             elif type(v) in [float, int, list]:
                 # Ints, floats, and lists get converted into records
                 if not is_multi:
-                    records.append(TimestreamRecord(
-                        measure_name=k,
-                        measure_value=v if type(v) in [float, int] else len(v),
-                        mstime=mstime
-                    ))
+                    records.append(
+                        TimestreamRecord(
+                            measure_name=k,
+                            measure_value=v if type(v) in [float, int] else len(v),
+                            mstime=mstime,
+                        )
+                    )
                 else:
-                    records.append({
-                        k:v
-                    })
+                    records.append({k: v})
             elif type(v) is str:
                 # Strings get converted into dimensions. Ignore empty strings
                 if v.strip():
@@ -383,6 +358,9 @@ class TimestreamLoader(TimestreamReader):
                     raise AssertionError(msg)
                 self._logger.warning(msg)
 
+        if not ignore_parse_error:
+            assert len(dimensions) >= 1, "Record must contain at least one dimension"
+
         # Recreate all records with dimensions attached and return
         if not is_multi:
             return [
@@ -390,15 +368,11 @@ class TimestreamLoader(TimestreamReader):
                     measure_name=r.measure_name,
                     measure_value=r.measure_value,
                     dimensions=dimensions,
-                    mstime=r.mstime
+                    mstime=r.mstime,
                 )
                 for r in records
             ]
         else:
             return [
-                TimestreamRecordMulti(
-                    measure_values=records,
-                    dimensions=dimensions,
-                    mstime=mstime
-                )
+                TimestreamRecordMulti(measure_values=records, dimensions=dimensions, mstime=mstime)
             ]
